﻿using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2024
{
    public class Day2 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var numSafeReports = 0;
            foreach (var level in this.GetSplitInput())
            {
                if (IsValidLevel(level.Split(" ").Select(x => int.Parse(x)).ToArray()))
                {
                    numSafeReports++;
                }
            }
            Answer = numSafeReports;
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            var numSafeReports = 0;
            foreach (var level in this.GetSplitInput())
            {
                var pLevel = level.Split(" ").Select(x => int.Parse(x)).ToArray();
                if (IsValidLevel(pLevel)) //If initially safe, increase the count
                {
                    numSafeReports++;
                }
                else
                {
                    for (int i = 0; i < pLevel.Length; i++)
                    {
                        //Remove a single value of each array and test again, if it ever works then consider valid
                        var tempLevel = pLevel.Where((_, index) => index != i).ToArray();
                        if (IsValidLevel(tempLevel))
                        {
                            numSafeReports++;
                            break;
                        }
                    }
                }
            }
            Answer = numSafeReports;
            return this._response;
        }

        private bool IsValidLevel(int[] levels)
        {
            char dir = 'U';
            for (int i = 0; i < levels.Length-1; i++)
            {
                var delta = Math.Abs(levels[i] - levels[i + 1]);
                if (delta < 1 || delta > 3)
                    return false;
                switch (dir)
                {
                    case 'U':
                        dir = levels[i] < levels[i + 1] ? 'I' : 'D';
                        break;
                    case 'I': //Record should be increasing
                        if (levels[i] >= levels[i + 1])
                            return false;
                        break;
                    case 'D': // Record should be decreasing
                        if (levels[i] <= levels[i + 1])
                            return false;
                        break;
                }
            }
            return true;
        }
    }
}
