﻿using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2024
{
    public class Day3 : AOCDay
    {
        private static string Operation = "mul(";
        protected override AOCResponse ExecutePartA()
        {
            var totalSum = 0;
            var row = string.Join("", this.GetSplitInput());
            foreach (var mulIndex in IndexesOfValue(row, Operation))
            {
                if (this._request.Version == AOCVersion.B //We only check enablement for Part B
                    && !IsEnabledAtIndex(row, mulIndex)) continue;
                var commaSplit = row.IndexOf(",", mulIndex + 1);
                if (commaSplit == -1) continue; //Means nothing was found
                var firstOperand = row.Substring(mulIndex, (commaSplit-mulIndex)).Replace(Operation, ""); //Remove 1 to get rid of the comma
                var endBracket = row.IndexOf(")", commaSplit + 1);
                if (endBracket == -1) continue; //Means nothing was found
                var secondOperand = row.Substring(commaSplit + 1, (endBracket - commaSplit) - 1);
                if (IsValidOperand(firstOperand) && IsValidOperand(secondOperand))
                {
                    Log($"{firstOperand} * {secondOperand}");
                    totalSum += int.Parse(firstOperand) * int.Parse(secondOperand);
                }
            }
            Answer = totalSum;
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            return ExecutePartA();
        }

        private bool IsValidOperand(string operand)
        {
            if (operand.Length > 3) return false;
            if (!operand.Replace(" ", "").Equals(operand)) return false;
            if (!int.TryParse(operand, out _)) return false;

            return true;
        }

        private bool IsEnabledAtIndex(string row, int index)
        {
            //Get the indexes of the entries, then take the higher index one indicating more recent status
            var enabledIndex = IndexesOfValue(row.Substring(0, index), "do()");
            var disabledIndex = IndexesOfValue(row.Substring(0, index), "don't()");

            if (!enabledIndex.Any())
            {
                return !disabledIndex.Any();
            }
            if (!disabledIndex.Any())
            {
                return true; //We assume that it is enabled if not set to disabled
            }
            return enabledIndex.Last() > disabledIndex.Last();
        }

        private List<int> IndexesOfValue(string row, string value)
        {
            var indexes = new List<int>();
            int index = row.IndexOf(value);

            while (index != -1)
            {
                indexes.Add(index); // Store the index
                index = row.IndexOf(value, index + 1); // Search from the next position
            }
            return indexes;
        }
    }
}
