﻿using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2024
{
    public class Day4 : AOCDay
    {
        private int[,] AllDirections = {
            {-1,  0}, //Up
            { 1,  0}, //Down
            { 0, -1}, //Left
            { 0,  1}, //Right
            {-1, -1}, //Diagonal up-left
            {-1,  1}, //Diagonal up-right
            { 1, -1}, //Diagonal down-left
            { 1,  1}  //Diagonal down-right
        };
        protected override AOCResponse ExecutePartA()
        {
            var input = this.GetSplitInput();

            //Create our grid
            var rows = input.Length;
            var columns = input.First().Length;
            var board = new char[rows, columns];
            for (int y = 0; y < rows; y++)
            {
                for (int x = 0; x < columns; x++)
                {
                    board[x, y] = input[x][y];
                }
            }

            //Parse our grid in all directions
            var numOfXmasWords = 0;
            for (int x = 0; x < board.GetLength(0); x++)
            {
                for (int y = 0; y < board.GetLength(1); y++)
                {
                    for (int d = 0; d < AllDirections.GetLength(0); d++)
                    {
                        int newRow = x;
                        int newCol = y;
                        string path = $"{board[x, y]}";

                        bool valid = true;
                        for (int step = 1; step < 4; step++)
                        {
                            newRow += AllDirections[d, 0];
                            newCol += AllDirections[d, 1];
                            if (newRow < 0 || newRow >= rows || newCol < 0 || newCol >= columns)
                            {
                                valid = false;
                                break;
                            }
                            path += board[newRow, newCol];
                        }

                        if (valid && path.Equals("XMAS"))
                        {
                            numOfXmasWords++;
                        }
                    }
                }
            }
            Answer = numOfXmasWords;
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            {
                var input = this.GetSplitInput();

                //Create our grid
                var rows = input.Length;
                var columns = input.First().Length;
                var board = new char[rows, columns];
                for (int y = 0; y < rows; y++)
                {
                    for (int x = 0; x < columns; x++)
                    {
                        board[x, y] = input[x][y];
                    }
                }

                //Parse our grid in all directions
                var numOfXmasWords = 0;
                for (int x = 0; x < board.GetLength(0); x++)
                {
                    for (int y = 0; y < board.GetLength(1); y++)
                    {
                        if (!board[x, y].Equals('A')) continue;
                        int[,] CrossedDirections = {
                            {-1, -1}, //Diagonal up-left
                            { 1,  1},  //Diagonal down-right
                            {-1,  1}, //Diagonal up-right
                            { 1, -1} //Diagonal down-left
                        };

                        var leftDiag = string.Empty;
                        var rightDiag = string.Empty;
                        for (int i = 0; i < CrossedDirections.GetLength(0); i++)
                        {
                            int newX = x + CrossedDirections[i, 0];
                            int newY = y + CrossedDirections[i, 1];

                            if (newX >= 0 && newX < rows && newY >= 0 && newY < columns)
                            {
                                if (i < 2)
                                    leftDiag += board[newX, newY];
                                else
                                    rightDiag += board[newX, newY];
                            }
                        }
                        if (leftDiag.Contains("M") && leftDiag.Contains("S") &&
                            rightDiag.Contains("M") && rightDiag.Contains("S"))
                        {
                            numOfXmasWords++;
                        }
                    }
                }
                Answer = numOfXmasWords;
                return this._response;
            }
        }
    }
}
