﻿using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Intrinsics.Arm;
using System.Threading.Tasks;

namespace AdventOfCode._2024
{
    public class Day5 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var input = this.SplitInputOnEmptyLine();
            var orderingRules = input[0].Select(x => new OrderingRule(x)).ToList();

            var validUpdates = new List<int[]>();
            foreach (var update in input[1])
            {
                var updateOrder = update.Split(",").Select(x => int.Parse(x)).ToArray();
                var applicableRules = orderingRules.Where(x => updateOrder.Contains(x.Min));
                var violation = false;
                foreach (var applicableRule in applicableRules)
                {
                    if (applicableRule.DoesViolateOrder(updateOrder))
                    {
                        violation = true;
                        break;
                    }
                }
                if (!violation)
                {
                    validUpdates.Add(updateOrder);
                }
            }

            var finalSum = 0;
            foreach (var validUpdate in validUpdates)
            {
                finalSum += validUpdate[validUpdate.Length / 2];
            }
            Answer = finalSum;
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            var input = this.SplitInputOnEmptyLine();
            var orderingRules = input[0].Select(x => new OrderingRule(x)).ToList();

            var invalidUpdates = new List<int[]>();
            foreach (var update in input[1])
            {
                var updateOrder = update.Split(",").Select(x => int.Parse(x)).ToArray();
                var applicableRules = orderingRules.Where(x => updateOrder.Contains(x.Min));
                var violation = false;
                foreach (var applicableRule in applicableRules)
                {
                    if (applicableRule.DoesViolateOrder(updateOrder))
                    {
                        violation = true;
                        break;
                    }
                }
                if (violation)
                {
                    invalidUpdates.Add(updateOrder);
                }
            }

            var finalSum = 0;
            foreach (var invalidUpdate in invalidUpdates)
            {
                var applicableRules = orderingRules.Where(x => invalidUpdate.Contains(x.Min) && invalidUpdate.Contains(x.Max));
                do
                {
                    foreach (var applicableRule in applicableRules)
                    {
                        if (applicableRule.DoesViolateOrder(invalidUpdate))
                        {
                            var first = Array.IndexOf(invalidUpdate, applicableRule.Min);
                            var second = Array.IndexOf(invalidUpdate, applicableRule.Max);
                            var swap = invalidUpdate[first];
                            invalidUpdate[first] = invalidUpdate[second];
                            invalidUpdate[second] = swap;
                        }
                    }
                } while (applicableRules.Any(x => x.DoesViolateOrder(invalidUpdate)));
                finalSum += invalidUpdate[invalidUpdate.Length / 2];
                Log(string.Join(",",invalidUpdate));
            }
            Answer = finalSum;
            return this._response;
        }
    }

    class OrderingRule
    {
        public int Min { get; set; }
        public int Max { get; set; }
        public OrderingRule(string rule)
        {
            var split = rule.Split('|');
            Min = int.Parse(split[0]);
            Max = int.Parse(split[1]);
        }

        public bool DoesViolateOrder(int[] pageOrdering)
        {
            var indexOfMin = Array.IndexOf(pageOrdering, Min);
            var indexOfMax = Array.IndexOf(pageOrdering, Max);
            if (indexOfMin == -1 || indexOfMax == -1)
                return false; //Means no page found in rule, should skip
            return indexOfMin > indexOfMax;
        }
    }
}
