﻿using AdventOfCode._2022.Day9;
using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Intrinsics.Arm;
using System.Threading.Tasks;

namespace AdventOfCode._2024
{
    public class Day6 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var input = this.GetSplitInput();
            //Create our grid
            var rows = input.Length;
            var columns = input.First().Length;
            var board = new char[rows, columns];
            Guard guard = null;
            for (int y = 0; y < rows; y++)
            {
                for (int x = 0; x < columns; x++)
                {
                    board[x, y] = input[x][y];
                    if (input[x][y] == '^')
                    {
                        guard = new Guard(x, y);
                    }
                }
            }

            while (guard.IsWithinBoard)
            {
                guard.Move(board);
            }
            Answer = guard.VisitedLocations.Count;
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            return this._response;
        }
    }
    class Guard
    {
        public bool IsWithinBoard {  get; set; }
        public HashSet<string> VisitedLocations;
        public Direction Direction { get; set; }
        public int X { get; set; }
        public int Y { get; set; }
        public Guard(int x, int y)
        {
            IsWithinBoard = true;
            Direction = Direction.U;
            X = x;
            Y = y;
            VisitedLocations = new HashSet<string>();
            MarkLocationAsVisited(); //Initial starting point
        }

        public void Move(char[,] board)
        {
            int newX = GetNewX();
            int newY = GetNewY();
            if (!board.IsWithinBoard(newX, newY))
            {
                IsWithinBoard = false;
                return;
            }
            else
            {
                while (board[newX, newY] == '#')
                {
                    Rotate();
                    newX = GetNewX();
                    newY = GetNewY();
                    if (!board.IsWithinBoard(newX, newY))
                    {
                        IsWithinBoard = false;
                        return;
                    }
                }
            }

            //if (newX < 0 || newY < 0
            //    || newX > board.GetLength(0) || newY > board.GetLength(1))
            //{
            //    IsWithinBoard = false;
            //    return;
            //}

            //do
            //{
            //    newX = GetNewX();
            //    newY = GetNewY();
            //    if (newX < 0 || newY < 0
            //        || newX > board.GetLength(0) || newY > board.GetLength(1))
            //    {
            //        IsWithinBoard = false;
            //        return;
            //    }

            //} while (board[newX, newY] == '#');

            board[X, Y] = 'X';
            X = newX;
            Y = newY;
            board[X, Y] = GetDirectionChar();
            MarkLocationAsVisited(); //Track each visited hex
            //TODO Check if it exceeds the board
        }

        private void MarkLocationAsVisited()
        {
            VisitedLocations.Add($"{X}-{Y}");
        }

        private int GetNewX()
        {
            if (Direction == Direction.U)
                return X - 1;
            if (Direction == Direction.D)
                return X + 1;
            return X;
        }

        private int GetNewY()
        {
            if (Direction == Direction.L)
                return Y - 1;
            if (Direction == Direction.R)
                return Y + 1;
            return Y;
        }

        private void Rotate()
        {
            if (Direction == Direction.U)
            {
                Direction = Direction.R;
            }else if (Direction == Direction.D)
            {
                Direction = Direction.L;
            }
            else if (Direction == Direction.L)
            {
                Direction = Direction.U;
            }
            else if (Direction == Direction.R)
            {
                Direction = Direction.D;
            }
        }

        private char GetDirectionChar()
        {
            switch (Direction)
            {
                case Direction.R:
                    return '>';
                case Direction.L:
                    return '<';
                case Direction.U:
                    return '^';
                case Direction.D:
                    return 'v';
            }
            throw new ArgumentException("Invalid direction");
        }
    }
}
