﻿using AdventOfCode._2022.Day9;
using AdventOfCode.Common;
using AdventOfCode.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Intrinsics.Arm;
using System.Threading.Tasks;

namespace AdventOfCode._2024
{
    public class Day7 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var sum = 0l;
            foreach (var line in this.GetSplitInput())
            {
                var to = new TestEquation(line, this._request.Version == AOCVersion.B);
                if (to.Evaluate())
                {
                    sum += to.Result;
                }
            }
            Answer = sum;
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            return ExecutePartA();
        }
    }

    class TestEquation
    {
        private char[] ValidOperators = new char[] { '+', '*' };
        public long Result { get; set; }
        private long[] Operands;
        public TestEquation(string input, bool isPartB)
        {
            var split = input.Split(':');
            Result = long.Parse(split[0]);
            var operands = split[1].Trim().Split(" ");
            Operands = operands.Select(x => long.Parse(x)).ToArray();
            if (isPartB)
            {
                ValidOperators = new char[] { '+', '*', '|' };
            }
        }

        public bool Evaluate()
        {
            var combinations = GenerateCombinations(Operands.Length - 1);
            foreach (var combination in combinations)
            {
                var equation = new List<string>();
                int operatorPos = 0;
                for (int i = 0; i < Operands.Length; i++)
                {
                    equation.Add(Operands[i].ToString());
                    if (operatorPos < combination.Length)
                        equation.Add(combination[operatorPos++].ToString());
                }
                var result = Compute(equation);
                if (result == Result)
                {
                    return true;
                }
            }
            return false;
        }

        private long Compute(List<string> equation)
        {
            var result = long.Parse(equation.First());
            for (int i = 1; i < equation.Count; i++)
            {
                if (equation[i] == "+")
                {
                    result += long.Parse(equation[++i]);
                }
                else if (equation[i] == "*")
                {
                    result *= long.Parse(equation[++i]);
                }else if (equation[i] == "|")
                {
                    result = long.Parse(result.ToString() + equation[++i]);
                }
            }
            return result;
        }

        private List<string> GenerateCombinations(int n)
        {
            var results = new List<string>();
            GenerateCombinationsRecursive(n, "", results);
            return results;
        }

        void GenerateCombinationsRecursive(int n,  string current, List<string> results)
        {
            if (current.Length == n)
            {
                results.Add(current);
                return;
            }

            foreach (var option in ValidOperators)
            {
                GenerateCombinationsRecursive(n, current + option, results);
            }
        }



    }
}
